import uuid
import json
from model import BaseModel
from peewee import IntegerField, CharField, ForeignKeyField, BooleanField

class Question(BaseModel):

    uuid = CharField(primary_key=True)
    question = CharField()
    used = BooleanField(default=False)
    taken = BooleanField(default=False)

    def __str__(self):
        return question

    @staticmethod
    def read_from_json(path_to_json):
        question_list = None
        with open(path_to_json, "r") as file:
            question_list = json.load(file)

        for q in question_list:
            question_string = q["question"]
            if "id" not in q or q["id"] is None:
                q["id"] = str(uuid.uuid4())
                
            question = Question.create(uuid=q["id"], question=question_string)
            answer_list = []
            for a in q["answers"]:
                answer_string = a["answer"]
                points = a["score"]
                if "id" not in a:
                    id = str(uuid.uuid4())
                else:
                    id = a["id"]

                answer = Answer.create(question=question, answer=answer_string, points=points, uuid=id)
                answer.save()
                answer_list.append(answer)
            # question.answers = answer_lis
            question.save()

    def get_answer_by_id(self, id):
        for answer in self.answers:
            if answer.uuid == id:
                return answer
        return None

    def _calculate_modfier(self):
        answer_amount = len(self.answers)
        return 6 - answer_amount

    def apply_modifier(self):
        modifier = self._calculate_modfier()
        for answer in self.answers:
            answer.modifier = modifier
        self._sort_answers_by_points()

    def _sort_answers_by_points(self):
        self.answers = sorted(self.answers, key=lambda answer: answer.score)

    def use(self):
        self.used = True
        

class Answer(BaseModel):

    answer = CharField()
    points = IntegerField(default=0)
    hidden = BooleanField(default=True)
    modifier = IntegerField(default=1)
    question = ForeignKeyField(Question, backref="answers")
    uuid = CharField()

    def hide(self):
        if not self.hidden:
            self.hidden = True

    def show(self):
        if self.hidden:
            self.hidden = False


    @property
    def score(self):
        return self.modifier * self.points

    @score.setter
    def score(self, score):
        pass

    @score.deleter
    def score(self):
        pass