from flask import Flask, render_template, redirect, request, Blueprint
import os

from model import Team, Game, Question
from views import set_current_view, send_reply, build_current_round_json, verify_request_data
from utils import ERROR_LOG
mgmt = Blueprint("management", "management")

TEAMLOGO_DIR = os.path.join("static", "teamlogos")

if not os.path.isdir(TEAMLOGO_DIR):
    os.mkdir(TEAMLOGO_DIR)

def _generate_teamlogo_path(filename):
    return os.path.join(TEAMLOGO_DIR, filename)

DEFAULT_LOGO = _generate_teamlogo_path("0_no_logo_00.png")

# Question.read_from_json("data/questions_35c3.json")

@mgmt.route("/frontend", methods=["GET"])
def render_frontend():
    return render_template("frontend.jinja")

@mgmt.route("/management", methods=["GET", "POST"])
def render_management_view():
    return send_reply(render_template("management.jinja", teamlist=Team.select(), gamelist=Game.select(), error_list=ERROR_LOG.fetch()))

@mgmt.route("/remove_game", methods=["POST"])
def remove_game():
    gamename = request.form.get("gamename")
    game = Game.get_or_none(gamename=gamename)

@mgmt.route("/create_game", methods=["POST"])
def create_game():
    teamname_left = request.form.get("leftteam")
    teamname_right = request.form.get("rightteam")

    if teamname_right is None:
        ERROR_LOG.add("Request is incomplete. Missing name of team-right")
    if teamname_left is None:
        ERROR_LOG.add("Request is incomplete. Missing name of team-left")

    gamename = request.form.get("gamename")
    if gamename is "":
        gamename = teamname_left + "<>" + teamname_right

    team_left = Team.get_or_none(name=teamname_left)
    team_right = Team.get_or_none(name=teamname_right)

    game = Game.create(name=gamename, team_left=team_left, team_right=team_right)
    game.save()

    return render_management_view()

@mgmt.route("/import_question", methods=["POST"])
def import_questin():
    if "question_json" in request.files:
        json_file = request.files["question_json"]
        path = os.path.join("static", json_file.filename)
        json_file.save(path)
        Question.read_from_json(path)

        return redirect("/management")

@mgmt.route("/create_team", methods=["POST"])
def create_team():
    teamname = request.form.get("teamname")
    custom_teamlogo_path = _generate_teamlogo_path(teamname + ".png")
    print(custom_teamlogo_path)

    if "teamlogo" in request.files:
        teamlogo = request.files["teamlogo"]
        path = _generate_teamlogo_path( teamlogo.filename)
        teamlogo.save(path)
        teamlogo = path
    elif os.path.exists(custom_teamlogo_path):
        teamlogo = custom_teamlogo_path
    else:
        teamlogo = DEFAULT_LOGO
    
    team = Team.create(name=teamname, logo_path=teamlogo)
    team.save()

    return redirect("/management")

@mgmt.route("/remove_team", methods=["POST"])
def remove_team():
    teamname = request.form.get("teamname")
    # teampool.remove(teamname)

    return redirect("/management")

@mgmt.route("/fe_intro", methods=["POST"])
def fe_intro():
    set_current_view(view="intro_screen")
    game = verify_request_data(Game, "name", request.form.get("gamename"), log_error=False)
    build_current_round_json(game)
    return send_reply()

@mgmt.route("/fe_running", methods=["POST"])
def fe_running():
    set_current_view(view="player_select")
    game = verify_request_data(Game, "name", request.form.get("gamename"), log_error=False)
    build_current_round_json(game)
    return send_reply()

@mgmt.route("/fe_final", methods=["POST"])
def fe_final():
    set_current_view(view="final_screen")
    game = verify_request_data(Game, "name", request.form.get("gamename"), log_error=False)
    build_current_round_json(game)
    return send_reply()